searchData={"items":[{"type":"module","title":"Unicode.Regex","doc":"Implements [Unicode regular expressions](http://unicode.org/reports/tr18/)\nby transforming them into regular expressions supported by\nthe Elixir Regex module.","ref":"Unicode.Regex.html"},{"type":"function","title":"Unicode.Regex.compile/2","doc":"Compiles a binary regular expression after\nexpanding any Unicode Sets.","ref":"Unicode.Regex.html#compile/2"},{"type":"function","title":"Arguments - Unicode.Regex.compile/2","doc":"* `string` is a regular expression in\n  string form\n\n* `options` is a string or a list which is\n  passed unchanged to `Regex.compile/2`.\n  The default is \"u\" meaning the regular\n  expression will operate in Unicode mode","ref":"Unicode.Regex.html#compile/2-arguments"},{"type":"function","title":"Returns - Unicode.Regex.compile/2","doc":"* `{:ok, regex}` or\n\n* `{:error, {message, index}}`","ref":"Unicode.Regex.html#compile/2-returns"},{"type":"function","title":"Notes - Unicode.Regex.compile/2","doc":"This function operates by splitting the string\nat the boundaries of Unicode Set markers which\nare:\n\n* Posix style: `[:` and `:]`\n* Perl style: `\\p{` and `}`\n\nThis parsing is naive meaning that is does not\ntake any character escaping into account when s\nplitting the string.","ref":"Unicode.Regex.html#compile/2-notes"},{"type":"function","title":"Example - Unicode.Regex.compile/2","doc":"iex> Unicode.Regex.compile(\"[:Zs:]\")\n    {:ok, ~r/[\\x{20}\\x{A0}\\x{1680}\\x{2000}-\\x{200A}\\x{202F}\\x{205F}\\x{3000}]/u}\n\n    iex> Unicode.Regex.compile(\"\\\\p{Zs}\")\n    {:ok, ~r/[\\x{20}\\x{A0}\\x{1680}\\x{2000}-\\x{200A}\\x{202F}\\x{205F}\\x{3000}]/u}\n\n    iex> Unicode.Regex.compile(\"[:ZZZZ:]\")\n    {:error, {~c\"POSIX named classes are supported only within a class\", 0}}","ref":"Unicode.Regex.html#compile/2-example"},{"type":"function","title":"Unicode.Regex.compile!/2","doc":"Compiles a binary regular expression after\ninterpolating any Unicode Sets.","ref":"Unicode.Regex.html#compile!/2"},{"type":"function","title":"Arguments - Unicode.Regex.compile!/2","doc":"* `string` is a regular expression in\n  string form.\n\n* `options` is a string or a list which is\n  passed unchanged to `Regex.compile/2`.\n  The default is \"u\" meaning the regular\n  expression will operate in Unicode mode","ref":"Unicode.Regex.html#compile!/2-arguments"},{"type":"function","title":"Returns - Unicode.Regex.compile!/2","doc":"* `regex` or\n\n* raises an exception","ref":"Unicode.Regex.html#compile!/2-returns"},{"type":"function","title":"Example - Unicode.Regex.compile!/2","doc":"iex> Unicode.Regex.compile!(\"[:Zs:]\")\n    ~r/[\\x{20}\\x{A0}\\x{1680}\\x{2000}-\\x{200A}\\x{202F}\\x{205F}\\x{3000}]/u","ref":"Unicode.Regex.html#compile!/2-example"},{"type":"function","title":"Unicode.Regex.expand_regex/1","doc":"Expand a Unicode Set regex string into a regex string\nsupported OTP's `:re` module.","ref":"Unicode.Regex.html#expand_regex/1"},{"type":"function","title":"Arguments - Unicode.Regex.expand_regex/1","doc":"* `string` is a regular expression in\n  string form.","ref":"Unicode.Regex.html#expand_regex/1-arguments"},{"type":"function","title":"Returns - Unicode.Regex.expand_regex/1","doc":"* A regex string that can be compiled by\n  the `:re` module.","ref":"Unicode.Regex.html#expand_regex/1-returns"},{"type":"macro","title":"Unicode.Regex.is_perl_set/1","doc":"","ref":"Unicode.Regex.html#is_perl_set/1"},{"type":"function","title":"Unicode.Regex.match?/3","doc":"Returns a boolean indicating whether there was a match or not\nwith a Unicode Set.","ref":"Unicode.Regex.html#match?/3"},{"type":"function","title":"Arguments - Unicode.Regex.match?/3","doc":"* `regex_string` is a regular expression in\n  string form.\n\n* `string` is any string against which\n  the regex match is executed\n\n* `options` is a string or a list which is\n  passed unchanged to `Regex.compile/2`.\n  The default is \"u\" meaning the regular\n  expression will operate in Unicode mode","ref":"Unicode.Regex.html#match?/3-arguments"},{"type":"function","title":"Returns - Unicode.Regex.match?/3","doc":"* a boolean indicating if there was a match or\n\n* raises an exception if `regex` is not\n  a valid regular expression.","ref":"Unicode.Regex.html#match?/3-returns"},{"type":"function","title":"Example - Unicode.Regex.match?/3","doc":"iex> Unicode.Regex.match?(\"[:Sc:]\", \"$\")\n    true","ref":"Unicode.Regex.html#match?/3-example"},{"type":"function","title":"Unicode.Regex.split_character_classes/1","doc":"Split a regex into character classes\nso that these can then be later compiled.","ref":"Unicode.Regex.html#split_character_classes/1"},{"type":"function","title":"Arguments - Unicode.Regex.split_character_classes/1","doc":"* `string` is a regular expression in\n  string form.","ref":"Unicode.Regex.html#split_character_classes/1-arguments"},{"type":"function","title":"Returns - Unicode.Regex.split_character_classes/1","doc":"* A list of string split at the\n  boundaries of unicode sets","ref":"Unicode.Regex.html#split_character_classes/1-returns"},{"type":"function","title":"Example - Unicode.Regex.split_character_classes/1","doc":"iex> Unicode.Regex.split_character_classes(\"This is [:Zs:] and more\")\n    [\"This is \", \"[:Zs:]\", \" and more\"]","ref":"Unicode.Regex.html#split_character_classes/1-example"},{"type":"module","title":"Unicode.Regex.Match","doc":"","ref":"Unicode.Regex.Match.html"},{"type":"module","title":"Unicode.Set","doc":"","ref":"Unicode.Set.html"},{"type":"module","title":"Usage - Unicode.Set","doc":"","ref":"Unicode.Set.html#module-usage"},{"type":"module","title":"Function guards - Unicode.Set","doc":"This is helpful in defining [function guards](https://hexdocs.pm/elixir/patterns-and-guards.html#guards). For example:\n```elixir\ndefmodule Guards do\n  require Unicode.Set\n\n  # Define a guard that checks if a codepoint is a unicode digit\n  defguard digit?(x) when Unicode.Set.match?(x, \"[[:Nd:]]\")\nend\n\ndefmodule MyModule do\n  require Guards\n\n  # Define a function using the previously defined guard\n  def my_function(< >) when Guards.digit?(x) do\n    IO.puts \"Its a digit!\"\n  end\n\n  # Define a guard directly on the function\n  def my_other_function_(< >) when Unicode.Set.match?(x, \"[[:Nd:]]\") do\n    IO.puts \"Its also a digit!\"\n  end\nend\n```","ref":"Unicode.Set.html#module-function-guards"},{"type":"module","title":"Generating compiled patterns for String matching - Unicode.Set","doc":"`String.split/3` and `String.replace/3` allow for patterns and [compiled patterns](http://erlang.org/doc/man/binary.html#compile_pattern-1) to be used with compiled patterns being the more performant approach.  Unicode Set supports the generation of patterns and compiled patterns:\n```elixir\niex> pattern = Unicode.Set.compile_pattern!(\"[[:digit:]]\")\niex> list = String.split(\"abc1def2ghi3jkl\", pattern)\n[\"abc\", \"def\", \"ghi\", \"jkl\"]\n```","ref":"Unicode.Set.html#module-generating-compiled-patterns-for-string-matching"},{"type":"module","title":"Generating NimbleParsec ranges - Unicode.Set","doc":"The parser generator [nimble_parsec](https://hex.pm/packages/nimble_parsec) allows a list of codepoint ranges as parameters to several combinators. Unicode Set can generate such ranges:\n```\niex> Unicode.Set.to_utf8_char!(\"[[^abcd][mnb]]\")\n[98, 109..110, {:not, 97..100}]\n```\nThis can be used as shown in the following example:\n```elixir\ndefmodule MyCombinators do\n  import NimbleParsec\n\n  @digit_list = Unicode.Set.to_utf8_char!(\"[[:digit:]]\")\n  def unicode_digit do\n    utf8_char(@digit_list)\n    |> label(\"a digit in any Unicode script\")\n  end\nend\n```","ref":"Unicode.Set.html#module-generating-nimbleparsec-ranges"},{"type":"module","title":"Compiling extended regular expressions - Unicode.Set","doc":"The `Regex` module supports a limited set of Unicode Sets. The `Unicode.Regex` module provides `compile/2` and `compile!/2` functions that have the same arguments and compatible functionality with `Regexp.compile/2` other that they pre-process the regular expression, expanding any Unicode Sets. This makes it simple to incorporate Unicode Sets in regular expressions.\n\nAll Unicode Sets are expanded, even those that are known to `Regex.compile/2` since the erlang `:re` module upon `Regex` is based does not always keep pace with Unicode releases.\n\nFor example:\n\n```elixir\niex> Unicode.Regex.compile(\"\\\\p{Zs}\")\n{:ok, ~r/[\\x{20}\\x{A0}\\x{1680}\\x{2000}-\\x{200A}\\x{202F}\\x{205F}\\x{3000}]/u}\n\niex> Unicode.Regex.compile(\"[:graphic:]\")\n{:ok,\n ~r/[\\x{20}-\\x{7E}\\x{A0}-\\x{AC}\\x{AE}-\\x{377}\\x{37A}-\\x{37F}...]/u}\n```","ref":"Unicode.Set.html#module-compiling-extended-regular-expressions"},{"type":"module","title":"Other Examples - Unicode.Set","doc":"These examples show how to combine sets (union, difference and intersection) to deliver a flexible targeting of the required match.\n\n```elixir\n# The character \"๓\" is the thai digit `1`\niex> Unicode.Set.match? ?๓, \"[[:digit:]]\"\ntrue\n\n# Set operations allow union, insersection and difference\n# This example matches on digits, but not the Thai script\niex> Unicode.Set.match? ?๓, \"[[:digit:]-[:thai:]]\"\nfalse\n```","ref":"Unicode.Set.html#module-other-examples"},{"type":"module","title":"Compile time parsing - Unicode.Set","doc":"As much work as possible is done at compile time in order to deliver good performance. The macro `Unicode.Set.match?/2` parses the unicode set, expands the require codepoints and generates guard clauses at compile time. The resulting code is a simple set of boolean operators that executes quickly at runtime.","ref":"Unicode.Set.html#module-compile-time-parsing"},{"type":"module","title":"Supported Unicode properties - Unicode.Set","doc":"This version of `Unicode Set` supports the following enumerable unicode properties in unicode sets:\n\n* `script` such as `[:script=arabic:]`, `\\p{script=arabic}` or `[:arabic:]`\n* `block` such as `[:block=sudanese:]`, `\\p{block=sudanese}`, `\\p{IsSudanese}` or `[:IsSudanese:]`\n* `general category` such as `[:Lu:]`, `\\p{Lu}`, `[:gc=Lu:]` or `[:general category=Lu:]`\n* `combining class` such as `[:ccc=230:]`\n\nIn addition, the following boolean properties are supported. These are expressed as `[:white space:]` or `\\p{White Space}`.\n\n\nProperty   | Property | Property | Property\n---------- | -------- | -------- | ----------\nalphabetic | ascii_hex_digit | bidi_control | cased\n changes_when_casemapped | changes_when_lowercased |  changes_when_titlecased | changes_when_uppercased\ndash | default_ignorable_code_point |  deprecated |  diacritic\nextender | grapheme_base |  grapheme_extend |  grapheme_link\nhex_digit | hyphen |  id_continue |  id_start\nideographic | ids_binary_operator |  ids_trinary_operator |  join_control\nlogical_order_exception | lowercase |  math |  noncharacter_code_point\nother_alphabetic | other_default_ignorable_code_point |  other_grapheme_extend |  other_id_continue\nother_id_start |  other_lowercase | other_math |  other_uppercase\npattern_syntax |  pattern_white_space | prepended_concatenation_mark |  quotation_mark\nradical |  regional_indicator | sentence_terminal |  soft_dotted\nterminal_punctuation |  unified_ideograph | uppercase |  variation_selector\nwhite_space |  xid_continue | xid_start | changes_when_casefolded\n\nIn all cases, property names and property values may include whitespace and mixed case notation.","ref":"Unicode.Set.html#module-supported-unicode-properties"},{"type":"module","title":"General Categories - Unicode.Set","doc":"Abbreviation\t      | Long Form\n------------------- | --------------------------------------\nL  |\tLetter\nLu |\tUppercase Letter\nLl |\tLowercase Letter\nLt |\tTitlecase Letter\nLm |\tModifier Letter\nLo |\tOther Letter\nM  |\tMark\nMn |\tNon-Spacing Mark\nMc |\tSpacing Combining Mark\nMe |\tEnclosing Mark\nN  |\tNumber\nNd |\tDecimal Digit Number\nNl |\tLetter Number\nNo |\tOther Number\nS  |\tSymbol\nSm |\tMath Symbol\nSc |\tCurrency Symbol\nSk |\tModifier Symbol\nSo |\tOther Symbol\nP  |\tPunctuation\nPc |\tConnector Punctuation\nPd |\tDash Punctuation\nPs |\tOpen Punctuation\nPe |\tClose Punctuation\nPi |\tInitial Punctuation\nPf |\tFinal Punctuation\nPo |\tOther Punctuation\nZ  |\tSeparator\nZs |\tSpace Separator\nZl |\tLine Separator\nZp |\tParagraph Separator\nC  |\tOther\nCc |\tControl\nCf |\tFormat\nCs |\tSurrogate\nCo |\tPrivate Use\nCn |\tUnassigned\n\nDerived Categories\t| Long Form\n------------------- | --------------------------------------\nAny                 | Any\tall code points\t[\\u{0}-\\u{10FFFF}]\nAssigned            | Assigned\tall assigned characters meaning\t`\\P{Cn}`\nASCII               | ASCII\tall ASCII characters\t[\\u{0}-\\u{7F}]","ref":"Unicode.Set.html#module-general-categories"},{"type":"module","title":"Compatibility Property Names - Unicode.Set","doc":"Property  | Unicode Category           | Comments\n--------- | -------------------------- | -----------\nalpha\t    | `\\p{Alphabetic}`           | Alphabetic includes more than gc = Letter. Note that combining marks (Me, Mn, Mc) are required for words of many languages. While they could be applied to non-alphabetics, their principal use is on alphabetics. Alphabetic should not be used as an approximation for word boundaries: see `word` below.\nlower\t    | `\\p{Lowercase}`\t             | Lowercase includes more than gc = Lowercase_Letter (Ll).\nupper\t    | `\\p{Uppercase}`\t             | Uppercase includes more than gc = Uppercase_Letter (Lu).\npunct\t    | `\\p{gc=Punctuation} \\p{gc=Symbol} - \\p{alpha}` | Punctuation and symbols.\ndigit     |\t`\\p{gc=Decimal_Number}`\t     | [0..9]\tNon-decimal numbers (like Roman numerals) are normally excluded.\nxdigit    | `\\p{gc=Decimal_Number} \\p{Hex_Digit}`\t| [0-9 A-F a-f]\tHex_Digit contains 0-9 A-F, fullwidth and halfwidth, upper and lowercase.\nalnum     |\t`\\p{alpha} \\p{digit}`\t       | Simple combination of other properties\nspace     |\t`\\p{Whitespace}`\t|\nblank\t    | `\\p{gc=Space_Separator} \\N{CHARACTER TABULATION}`\t| \"horizontal\" whitespace: space separators plus U+0009 tab.\ncntrl\t    | `\\p{gc=Control} `            | The characters in \\p{gc=Format} share some, but not all aspects of control characters. Many format characters are required in the representation of plain text.\ngraph\t    | `[^\\p{space} \\p{gc=Control} \\p{gc=Surrogate} \\p{gc=Unassigned}]`\t| Warning: the set shown here is defined by excluding space, controls, and so on with ^.\nprint\t    | `\\p{graph} \\p{blank} -- \\p{cntrl}`\t| Includes graph and space-like characters.\nword      | `\\p{alpha} \\p{gc=Mark} \\p{digit} \\p{gc=Connector_Punctuation} \\p{Join_Control}`\t|\tThis is only an approximation to Word Boundaries. The Connector Punctuation is added in for programming language identifiers, thus adding `_` and similar characters.","ref":"Unicode.Set.html#module-compatibility-property-names"},{"type":"module","title":"Additional Derived properties - Unicode.Set","doc":"In addition to the Unicode properties, some additional properties are also defined for convenience. These properties related to quote marks and are:\n\n* `quote_mark`\n* `quote_mark_left`\n* `quote_mark_right`\n* `quote_mark_ambidextrous`\n* `quote_mark_single`\n* `quote_mark_double`\n\nAs above these properties can be expressed in mixed case with spaces and underscores inserted for readability.  They can be used in the same way as any Unicode property name.","ref":"Unicode.Set.html#module-additional-derived-properties"},{"type":"module","title":"Example Unicode Sets - Unicode.Set","doc":"Here are a few examples of sets. Although elements of the syntax appear similar to regular expressions, unicode sets only expresses one or more ranges of unicode codepoints.\n\nPattern\t              | Description\n--------------------- | -----------------------------------------------------------\n`[a-z]`               | The lower case letters `a` through `z`\n`[abc123]`            | The six characters `a,b,c,1,2` and `3`\n`[\\p{Letter}]`        | All characters with the Unicode General Category of Letter","ref":"Unicode.Set.html#module-example-unicode-sets"},{"type":"module","title":"String Values - Unicode.Set","doc":"In addition to being a set of characters (of Unicode code points), a UnicodeSet may also contain string values. Conceptually, the UnicodeSet is always a set of strings, not a set of characters, although in many common use cases the strings are all of length one, which reduces to being a set of characters.\n\nThis concept can be confusing when first encountered, probably because similar set constructs from other environments (regular expressions) can only contain characters.","ref":"Unicode.Set.html#module-string-values"},{"type":"module","title":"Unicode Set Patterns - Unicode.Set","doc":"Patterns are a series of characters bounded by square brackets that contain lists of characters and Unicode property sets. Lists are a sequence of characters that may have ranges indicated by a '-' between two characters, as in \"a-z\". The sequence specifies the range of all characters from the left to the right, in Unicode order. For example, `[a c d-f m]` is equivalent to `[a c d e f m]`. Whitespace can be freely used for clarity as `[a c d-f m]` means the same as `[acd-fm]`.\n\nUnicode property sets are specified by a Unicode property, such as [:Letter:]. For a list of supported properties, see the [Properties](#supported-unicode-properties) section. For details on the use of short vs. long property and property value names, see the end of this section. The syntax for specifying the property names is an extension of either POSIX or Perl syntax with the addition of `=value`. For example, you can match letters by using the POSIX syntax `[:Letter:]`, or by using the Perl-style syntax `\\p{Letter}`. The type can be omitted for the `Category` and `Script` properties, but is required for other properties.\n\nThe table below shows the two kinds of syntax: POSIX and Perl style. Also, the table shows the \"Negative\", which is a property that excludes all characters of a given kind. For example, `[:^Letter:]` matches all characters that are not `[:Letter:]`.\n\nStyle              | Positive\t        | Negative\n------------------ | ---------------- | ----------------\nPOSIX-style Syntax | [:type=value:]   |\t[:^type=value:]\nPerl-style Syntax  |\t\\p{type=value}\t| \\P{type=value}\n\nThese following low-level lists or properties then can be freely combined with the normal set operations (union, inverse, difference, and intersection):\n\nExample\t                       | Meaning\n------------------------------ | -----------------------------------------------------------\n`A B\t[[:letter:] [:number:]]` | To union two sets A and B, simply concatenate them\n`A & B\t[[:letter:] & [a-z]]`  | To intersect two sets A and B, use the '&' operator.\n`A - B\t[[:letter:] - [a-z]]`\t | To take the set-difference of two sets A and B, use the '-' operator.\n`[^A]\t[^a-z]`\t                 | To invert a set A, place a `^` immediately after the opening `[`. Note that the complement only affects code points, not string values. In any other location, the `^` does not have a special meaning.","ref":"Unicode.Set.html#module-unicode-set-patterns"},{"type":"module","title":"Precedence - Unicode.Set","doc":"The binary operators of union, intersection, and set-difference have equal precedence and bind left-to-right. Thus the following are equivalent:\n\n* `[[:letter:] - [a-z] [:number:] & [\\u0100-\\u01FF]]`\n* `[[[[[:letter:] - [a-z]] [:number:]] & [\\u0100-\\u01FF]]`\n\nAnother example is that the set `[[ace][bdf] - [abc][def]]` is not the empty set, but instead the set `[def]`. That is because the syntax corresponds to the following UnicodeSet operations:\n\n1. start with `[ace]`\n2. union `[bdf]`  -- we now have `[abcdef]`\n3. subtract `[abc]` -- we now have `[def]`\n4. union `[def]` -- no effect, we still have `[def]`\n\nThis only really matters where there are the difference and intersection operations, as the union operation is commutative. To make sure that the - is the main operator, add brackets to group the operations as desired, such as `[[ace][bdf] - [[abc][def]]]`.\n\nAnother caveat with the `&` and `-` operators is that they operate between sets. That is, they must be immediately preceded and immediately followed by a set. For example, the pattern `[[:Lu:]-A]` is illegal, since it is interpreted as the set [:Lu:] followed by the incomplete range -A. To specify the set of uppercase letters except for `A`, enclose the `A` in a set: `[[:Lu:]-[A]]`.","ref":"Unicode.Set.html#module-precedence"},{"type":"module","title":"Examples - Unicode.Set","doc":"* `[a]`\tThe set containing 'a'\n* `[a-z]`\tThe set containing 'a' through 'z' and all letters in between, in Unicode order\n* `[^a-z]`\tThe set containing all characters but 'a' through 'z', that is, U+0000 through 'a'-1 and 'z'+1 through U+FFFF\n* `[[pat1][pat2]]`\tThe union of sets specified by pat1 and pat2\n* `[[pat1]&[pat2]]`\tThe intersection of sets specified by pat1 and pat2\n* `[[pat1]-[pat2]]`\tThe asymmetric difference of sets specified by pat1 and pat2\n* `[:Lu:]`\tThe set of characters belonging to the given Unicode category; in this case, Unicode uppercase letters. The long form for this is `[:UppercaseLetter:]`.\n* `[:L:]`\tThe set of characters belonging to all Unicode categories starting with 'L', that is, `[[:Lu:][:Ll:][:Lt:][:Lm:][:Lo:]]`. The long form for this is `[:Letter:]`.","ref":"Unicode.Set.html#module-examples"},{"type":"module","title":"String Values in Sets - Unicode.Set","doc":"String values are enclosed in `{`curly brackets`}`.\n\nSet expression\t    | Description\n------------------- | --------------------------------------\n`[abc{def}]`\t      | A set containing four members, the single characters a, b and c, and the string “def”\n`[{abc}{def}]`      |\tA set containing two members, the string “abc” and the string “def”.\n`[{a}{b}{c}][abc]`\t| These two sets are equivalent. Each contains three items, the three individual characters `a`, `b` and `c`. A `{string}` containing a single character is equivalent to that same character specified in any other way.","ref":"Unicode.Set.html#module-string-values-in-sets"},{"type":"module","title":"Character Quoting and Escaping in Unicode Set Patterns - Unicode.Set","doc":"","ref":"Unicode.Set.html#module-character-quoting-and-escaping-in-unicode-set-patterns"},{"type":"module","title":"Single Quote - Unicode.Set","doc":"Two single quotes represents a single quote, either inside or outside single quotes.\n\nText within single quotes is not interpreted in any way (except for two adjacent single quotes). It is taken as literal text (special characters become non-special).\n\nThese quoting conventions for ICU UnicodeSets differ from those of regular expression character set expressions. In regular expressions, single quotes have no special meaning and are treated like any other literal character.","ref":"Unicode.Set.html#module-single-quote"},{"type":"module","title":"Backslash Escapes - Unicode.Set","doc":"Outside of single quotes, certain backslashed characters have special meaning. Note that these are escapes processed by Unicode Set (this library) and therefore require `\\\\\\\\` to be entered as a prefix. [Elixir also provides similar escapes](https://elixir-lang.org/getting-started/sigils.html#interpolation-and-escaping-in-sigils) as native part of its string processing and Elixir's escapes are to be preferred where possible.\n\nEscape         | Description\n-------------- | -------------------------------------------------\n\\uhhhh\t       | Exactly 4 hex digits; h in [0-9A-Fa-f]\n\\Uhhhhhhhh\t   | Exactly 8 hex digits\n\\xhh\t         | 1-2 hex digits\n\nCertain other escapes are native to Elixir and are applicable in Unicode Sets they are in any Elixir string:\n\nEscape         | Description\n-------------- | -------------------------------------------------\n\\a\t           | U+0007 (BELL)\n\\b\t           | U+0008 (BACKSPACE)\n\\t\t           | U+0009 (HORIZONTAL TAB)\n\\n\t           | U+000A (LINE FEED)\n\\v\t           | U+000B (VERTICAL TAB)\n\\f\t           | U+000C (FORM FEED)\n\\r\t           | U+000D (CARRIAGE RETURN)\n\\\\\t           | U+005C (BACKSLASH)\n\\xDD           | represents a single byte in hexadecimal (such as `\\x13`)\n\\uDDDD and \\u{D...} | represents a Unicode codepoint in hexadecimal (such as `\\u{1F600}`)\n\nAnything else following a backslash is mapped to itself, except in an environment where it is defined to have some special meaning. For example, `\\p{Lu}` is the set of uppercase letters in a Unicode Set.\n\nAny character formed as the result of a backslash escape loses any special meaning and is treated as a literal. In particular, note that `\\u` and `\\U` escapes create literal characters.","ref":"Unicode.Set.html#module-backslash-escapes"},{"type":"module","title":"Whitespace - Unicode.Set","doc":"Whitespace (as defined by the specification) is ignored unless it is quoted or backslashed.","ref":"Unicode.Set.html#module-whitespace"},{"type":"module","title":"Property Values - Unicode.Set","doc":"The following property value variants are recognized:\n\nFormat\t  | Example                           | Description\n--------- | --------------------------------- | ----------------------------------------------\nshort\t    | Lu                                | omits the type (used to prevent ambiguity and only allowed with the Category and Script properties)\nmedium\t  | gc=Lu                             | uses an abbreviated type and value\nlong\t    | General_Category=Uppercase_Letter | uses a full type and value\n\nIf the type or value is omitted, then the equals sign is also omitted. The short style is only\nused for Category and Script properties because these properties are very common and their omission is unambiguous.\n\nIn actual practice, you can mix type names and values that are omitted, abbreviated, or full. For example, if Category=Unassigned you could use what is in the table explicitly, `\\p{gc=Unassigned}`, `\\p{Category=Cn}`, or `\\p{Unassigned}`.\n\nWhen these are processed, case and whitespace are ignored so you may use them for clarity, if desired. For example, `\\p{Category = Uppercase Letter}` or `\\p{Category = uppercase letter}`.","ref":"Unicode.Set.html#module-property-values"},{"type":"type","title":"Unicode.Set.character_range/0","doc":"","ref":"Unicode.Set.html#t:character_range/0"},{"type":"type","title":"Unicode.Set.codepoint/0","doc":"","ref":"Unicode.Set.html#t:codepoint/0"},{"type":"type","title":"Unicode.Set.codepoint_range/0","doc":"","ref":"Unicode.Set.html#t:codepoint_range/0"},{"type":"function","title":"Unicode.Set.compile_pattern/1","doc":"Transforms a Unicode Set into a compiled\npattern that can be used with `String.split/3`\nand `String.replace/3`.\n\n[Compiled patterns](http://erlang.org/doc/man/binary.html#compile_pattern-1)\ncan be the more performant when matching strings.","ref":"Unicode.Set.html#compile_pattern/1"},{"type":"function","title":"Arguments - Unicode.Set.compile_pattern/1","doc":"* `unicode_set` is a string representation\n  of a Unicode Set","ref":"Unicode.Set.html#compile_pattern/1-arguments"},{"type":"function","title":"Returns - Unicode.Set.compile_pattern/1","doc":"* `{:ok, compiled_pattern}` or\n\n* `{:error, {exception, reason}}`","ref":"Unicode.Set.html#compile_pattern/1-returns"},{"type":"function","title":"Example - Unicode.Set.compile_pattern/1","doc":"iex> pattern = Unicode.Set.compile_pattern(\"[[:digit:]]\")\n    {:ok, {:ac, #Reference<0.2927979228.2367029250.255911>}}\n    iex> String.split(\"abc1def2ghi3jkl\", pattern)\n    [\"abc\", \"def\", \"ghi\", \"jkl\"]","ref":"Unicode.Set.html#compile_pattern/1-example"},{"type":"function","title":"Unicode.Set.compile_pattern!/1","doc":"Transforms a Unicode Set into a compiled\npattern that can be used with `String.split/3`\nand `String.replace/3`. Raises an exception on\nerror.\n\n[Compiled patterns](http://erlang.org/doc/man/binary.html#compile_pattern-1)\ncan be the more performant when matching strings.","ref":"Unicode.Set.html#compile_pattern!/1"},{"type":"function","title":"Arguments - Unicode.Set.compile_pattern!/1","doc":"* `unicode_set` is a string representation\n  of a Unicode Set","ref":"Unicode.Set.html#compile_pattern!/1-arguments"},{"type":"function","title":"Returns - Unicode.Set.compile_pattern!/1","doc":"* `compiled_pattern` or\n\n* raises an exception.","ref":"Unicode.Set.html#compile_pattern!/1-returns"},{"type":"function","title":"Example - Unicode.Set.compile_pattern!/1","doc":"iex> pattern = Unicode.Set.compile_pattern!(\"[[:digit:]]\")\n    {:ac, #Reference<0.2927979228.2367029250.255911>}\n    iex> String.split(\"abc1def2ghi3jkl\", pattern)\n    [\"abc\", \"def\", \"ghi\", \"jkl\"]","ref":"Unicode.Set.html#compile_pattern!/1-example"},{"type":"type","title":"Unicode.Set.generated_match/0","doc":"","ref":"Unicode.Set.html#t:generated_match/0"},{"type":"macro","title":"Unicode.Set.match?/2","doc":"Returns a boolean based upon whether `var`\nmatches the provided `unicode_set`.","ref":"Unicode.Set.html#match?/2"},{"type":"macro","title":"Arguments - Unicode.Set.match?/2","doc":"* `var` is any integer variable (since codepoints\n  are integers)\n\n* `unicode_set` is a binary representation of\n  a unicode set. An exception will be raised if `unicode_set`\n  is not a compile time binary","ref":"Unicode.Set.html#match?/2-arguments"},{"type":"macro","title":"Returns - Unicode.Set.match?/2","doc":"* `true` or `false`","ref":"Unicode.Set.html#match?/2-returns"},{"type":"macro","title":"Examples - Unicode.Set.match?/2","doc":"* `Unicode.Set.match?/2` can be used with `defguard/1`.\n  For example:\n\n```elixir\ndefguard is_lower(codepoint) when Unicode.Set.match?(codepoint, \"[[:Lu:]]\")\n```\n* Or as a guard clause itself:\n\n```elixir\ndef my_function(< >)\n  when Unicode.Set.match?(codepoint, \"[[:Lu:]]\")\n```","ref":"Unicode.Set.html#match?/2-examples"},{"type":"type","title":"Unicode.Set.nimble_list/0","doc":"","ref":"Unicode.Set.html#t:nimble_list/0"},{"type":"type","title":"Unicode.Set.nimble_range/0","doc":"","ref":"Unicode.Set.html#t:nimble_range/0"},{"type":"type","title":"Unicode.Set.operation/0","doc":"","ref":"Unicode.Set.html#t:operation/0"},{"type":"type","title":"Unicode.Set.operator/0","doc":"","ref":"Unicode.Set.html#t:operator/0"},{"type":"function","title":"Unicode.Set.parse/1","doc":"","ref":"Unicode.Set.html#parse/1"},{"type":"function","title":"Unicode.Set.parse!/1","doc":"","ref":"Unicode.Set.html#parse!/1"},{"type":"function","title":"Unicode.Set.parse_and_reduce/1","doc":"Parses a unicode set and expands the\nset expressions then compacts the\ncharacter ranges.","ref":"Unicode.Set.html#parse_and_reduce/1"},{"type":"function","title":"Unicode.Set.parse_and_reduce!/1","doc":"","ref":"Unicode.Set.html#parse_and_reduce!/1"},{"type":"type","title":"Unicode.Set.range/0","doc":"","ref":"Unicode.Set.html#t:range/0"},{"type":"type","title":"Unicode.Set.range_list/0","doc":"","ref":"Unicode.Set.html#t:range_list/0"},{"type":"type","title":"Unicode.Set.state/0","doc":"","ref":"Unicode.Set.html#t:state/0"},{"type":"type","title":"Unicode.Set.string_range/0","doc":"","ref":"Unicode.Set.html#t:string_range/0"},{"type":"type","title":"Unicode.Set.t/0","doc":"","ref":"Unicode.Set.html#t:t/0"},{"type":"function","title":"Unicode.Set.to_pattern/1","doc":"Transforms a Unicode Set into a pattern\nthat can be used with `String.split/3`\nand `String.replace/3`.","ref":"Unicode.Set.html#to_pattern/1"},{"type":"function","title":"Arguments - Unicode.Set.to_pattern/1","doc":"* `unicode_set` is a string representation\n  of a Unicode Set","ref":"Unicode.Set.html#to_pattern/1-arguments"},{"type":"function","title":"Returns - Unicode.Set.to_pattern/1","doc":"* `{:ok, pattern}` or\n\n* `{:error, {exception, reason}}`","ref":"Unicode.Set.html#to_pattern/1-returns"},{"type":"function","title":"Example - Unicode.Set.to_pattern/1","doc":"iex> pattern = Unicode.Set.to_pattern \"[[:digit:]]\"\n    {:ok,\n     [\"0\", \"1\", \"2\", \"3\", \"4\", \"5\", \"6\", \"7\", \"8\", \"9\", \"٠\", \"١\", \"٢\", \"٣\",\n      \"٤\", \"٥\", \"٦\", \"٧\", \"٨\", \"٩\", \"۰\", \"۱\", \"۲\", \"۳\", \"۴\", \"۵\", \"۶\",\n      \"۷\", \"۸\", \"۹\", \"߀\", \"߁\", \"߂\", \"߃\", \"߄\", \"߅\", \"߆\", \"߇\", \"߈\", \"߉\",\n      \"०\", \"१\", \"२\", \"३\", \"४\", \"५\", \"६\", \"७\", ...]}","ref":"Unicode.Set.html#to_pattern/1-example"},{"type":"function","title":"Unicode.Set.to_pattern!/1","doc":"Transforms a Unicode Set into a pattern\nthat can be used with `String.split/3`\nand `String.replace/3`.","ref":"Unicode.Set.html#to_pattern!/1"},{"type":"function","title":"Arguments - Unicode.Set.to_pattern!/1","doc":"* `unicode_set` is a string representation\n  of a Unicode Set","ref":"Unicode.Set.html#to_pattern!/1-arguments"},{"type":"function","title":"Returns - Unicode.Set.to_pattern!/1","doc":"* `pattern` or\n\n* raises an exception","ref":"Unicode.Set.html#to_pattern!/1-returns"},{"type":"function","title":"Example - Unicode.Set.to_pattern!/1","doc":"iex> pattern = Unicode.Set.to_pattern \"[[:digit:]]\"\n    [\"0\", \"1\", \"2\", \"3\", \"4\", \"5\", \"6\", \"7\", \"8\", \"9\", \"٠\", \"١\", \"٢\", \"٣\",\n     \"٤\", \"٥\", \"٦\", \"٧\", \"٨\", \"٩\", \"۰\", \"۱\", \"۲\", \"۳\", \"۴\", \"۵\", \"۶\",\n      \"۷\", \"۸\", \"۹\", \"߀\", \"߁\", \"߂\", \"߃\", \"߄\", \"߅\", \"߆\", \"߇\", \"߈\", \"߉\"\n     \"०\", \"१\", \"२\", \"३\", \"४\", \"५\", \"६\", \"७\", ...]","ref":"Unicode.Set.html#to_pattern!/1-example"},{"type":"function","title":"Unicode.Set.to_regex_string/1","doc":"Transforms a Unicode Set into a regex\nstring that can be used as an argument\nto `Regex.compile/1`.","ref":"Unicode.Set.html#to_regex_string/1"},{"type":"function","title":"Arguments - Unicode.Set.to_regex_string/1","doc":"* `unicode_set` is a string representation\n  of a Unicode Set","ref":"Unicode.Set.html#to_regex_string/1-arguments"},{"type":"function","title":"Returns - Unicode.Set.to_regex_string/1","doc":"* `{:ok, regex_string}` or\n\n* `{:error, {exception, reason}}`","ref":"Unicode.Set.html#to_regex_string/1-returns"},{"type":"function","title":"Example - Unicode.Set.to_regex_string/1","doc":"iex> Unicode.Set.to_regex_string \"[[:Zs]-[ ]]\"\n    {:ok, \"[\\x{3A}\\x{5A}\\x{73}]\"}","ref":"Unicode.Set.html#to_regex_string/1-example"},{"type":"function","title":"Unicode.Set.to_regex_string!/1","doc":"Transforms a Unicode Set into a regex\nstring that can be used as an argument\nto `Regex.compile/1`.","ref":"Unicode.Set.html#to_regex_string!/1"},{"type":"function","title":"Arguments - Unicode.Set.to_regex_string!/1","doc":"* `unicode_set` is a string representation\n  of a Unicode Set","ref":"Unicode.Set.html#to_regex_string!/1-arguments"},{"type":"function","title":"Returns - Unicode.Set.to_regex_string!/1","doc":"* `regex_string` or\n\n* raises an exception","ref":"Unicode.Set.html#to_regex_string!/1-returns"},{"type":"function","title":"Example - Unicode.Set.to_regex_string!/1","doc":"iex> Unicode.Set.to_regex_string \"[[:Zs]-[ ]]\"\n    {:ok, \"[\\x{3A}\\x{5A}\\x{73}]\"}","ref":"Unicode.Set.html#to_regex_string!/1-example"},{"type":"function","title":"Unicode.Set.to_utf8_char/1","doc":"Transforms a Unicode Set into a list of\ncodepoints that can be used with\n[nimble_parsec](https://hex.pm/packages/nimble_parsec).\n\nTHe list of codepoints can be used as an\nargument to `NimbleParsec.utf8_char/1`.","ref":"Unicode.Set.html#to_utf8_char/1"},{"type":"function","title":"Arguments - Unicode.Set.to_utf8_char/1","doc":"* `unicode_set` is a string representation\n  of a Unicode Set","ref":"Unicode.Set.html#to_utf8_char/1-arguments"},{"type":"function","title":"Returns - Unicode.Set.to_utf8_char/1","doc":"* `{:ok, list_of_codepints}` or\n\n* `{:error, {exception, reason}}`","ref":"Unicode.Set.html#to_utf8_char/1-returns"},{"type":"function","title":"Example - Unicode.Set.to_utf8_char/1","doc":"iex> pattern = Unicode.Set.to_utf8_char \"[[:digit:]-[:Zs]]\"\n    {:ok,\n     [48..57, 1632..1641, 1776..1785, 1984..1993, 2406..2415, 2534..2543,\n      2662..2671, 2790..2799, 2918..2927, 3046..3055, 3174..3183, 3302..3311,\n      3430..3439, 3558..3567, 3664..3673, 3792..3801, 3872..3881, 4160..4169,\n      4240..4249, 6112..6121, 6160..6169, 6470..6479, 6608..6617, 6784..6793,\n      6800..6809, 6992..7001, 7088..7097, 7232..7241, 7248..7257, 42528..42537,\n      43216..43225, 43264..43273, 43472..43481, 43504..43513, 43600..43609,\n      44016..44025, 65296..65305, 66720..66729, 68912..68921, 69734..69743,\n      69872..69881, 69942..69951, 70096..70105, 70384..70393, 70736..70745,\n      70864..70873, 71248..71257, 71360..71369, ...]}","ref":"Unicode.Set.html#to_utf8_char/1-example"},{"type":"function","title":"Unicode.Set.to_utf8_char!/1","doc":"Transforms a Unicode Set into a list of\ncodepoints that can be used with\n[nimble_parsec](https://hex.pm/packages/nimble_parsec).\n\nTHe list of codepoints can be used as an\nargument to `NimbleParsec.utf8_char/1`.","ref":"Unicode.Set.html#to_utf8_char!/1"},{"type":"function","title":"Arguments - Unicode.Set.to_utf8_char!/1","doc":"* `unicode_set` is a string representation\n  of a Unicode Set","ref":"Unicode.Set.html#to_utf8_char!/1-arguments"},{"type":"function","title":"Returns - Unicode.Set.to_utf8_char!/1","doc":"* `list_of_codepints` or\n\n* raises an exception","ref":"Unicode.Set.html#to_utf8_char!/1-returns"},{"type":"function","title":"Example - Unicode.Set.to_utf8_char!/1","doc":"iex> pattern = Unicode.Set.to_utf8_char! \"[[:digit:]-[:Zs]]\"\n    [48..57, 1632..1641, 1776..1785, 1984..1993, 2406..2415, 2534..2543,\n     2662..2671, 2790..2799, 2918..2927, 3046..3055, 3174..3183, 3302..3311,\n     3430..3439, 3558..3567, 3664..3673, 3792..3801, 3872..3881, 4160..4169,\n     4240..4249, 6112..6121, 6160..6169, 6470..6479, 6608..6617, 6784..6793,\n     6800..6809, 6992..7001, 7088..7097, 7232..7241, 7248..7257, 42528..42537,\n     43216..43225, 43264..43273, 43472..43481, 43504..43513, 43600..43609,\n     44016..44025, 65296..65305, 66720..66729, 68912..68921, 69734..69743,\n     69872..69881, 69942..69951, 70096..70105, 70384..70393, 70736..70745,\n     70864..70873, 71248..71257, 71360..71369, ...]}","ref":"Unicode.Set.html#to_utf8_char!/1-example"},{"type":"module","title":"Unicode.Set.Operation","doc":"Functions to operate on Unicode sets:\n\n* Intersection\n* Difference\n* Union\n* Inversion","ref":"Unicode.Set.Operation.html"},{"type":"function","title":"Unicode.Set.Operation.combine/1","doc":"Combines all the ranges into a single list\n\nThis function is called iff the Unicode\nSets are formed by unions only. If\nthe set operations of intersection or\ndifference are present then the ranges\nwill need to be expanded via `expand/1`.","ref":"Unicode.Set.Operation.html#combine/1"},{"type":"function","title":"Unicode.Set.Operation.compact_ranges/1","doc":"Compact overlapping and adjacent ranges","ref":"Unicode.Set.Operation.html#compact_ranges/1"},{"type":"function","title":"Unicode.Set.Operation.complement/1","doc":"Returns the complement (inverse) of a set.","ref":"Unicode.Set.Operation.html#complement/1"},{"type":"function","title":"Unicode.Set.Operation.difference/2","doc":"Removes one list of 2-tuples\nrepresenting Unicode codepoints from\nanother.\n\nReturns the first list of codepoint\nranges minus the codepoints in the second\nlist.","ref":"Unicode.Set.Operation.html#difference/2"},{"type":"function","title":"Unicode.Set.Operation.expand/1","doc":"Expand takes a reduced AST and expands\nit into a single list of codepoint tuples.","ref":"Unicode.Set.Operation.html#expand/1"},{"type":"function","title":"Unicode.Set.Operation.expand_string_range/1","doc":"","ref":"Unicode.Set.Operation.html#expand_string_range/1"},{"type":"function","title":"Unicode.Set.Operation.expand_string_ranges/1","doc":"Expand string ranges like `{ab}-{cd}`","ref":"Unicode.Set.Operation.html#expand_string_ranges/1"},{"type":"function","title":"Unicode.Set.Operation.has_difference_or_intersection?/1","doc":"Returns a boolean indicating whether the given\nAST includes set operations intersection or\ndifference.\n\nWhen these operations exist then all ranges - including\n`^` ranges needs to be expanded.  If there are no\nintersections or differences then the `^` ranges can\nbe directly translated to guard clauses or a list of\nelixir ranges.","ref":"Unicode.Set.Operation.html#has_difference_or_intersection?/1"},{"type":"function","title":"Unicode.Set.Operation.intersect/2","doc":"Returns the intersection of two lists of\n2-tuples representing codepoint ranges.\n\nThe result is a single list of codepoint\nranges that represents the common codepoints\nin the two lists.","ref":"Unicode.Set.Operation.html#intersect/2"},{"type":"function","title":"Unicode.Set.Operation.reduce/1","doc":"Reduces all sets, properties and ranges to a list\nof 2-tuples expressing a range of codepoints.\n\nIt can return one of two forms\n\n`[{:in, [tuple_list]}]` for an inclusion list\n\n`[{:not_in, [tuple_list]}]` for an exclusion list\n\nor a combination of both.\n\nAttempts are made to preserve `:not_in` clauses\nas long as possible since many uses, like regexes\nand `nimble_parsec` can consume `:not_in` style\nranges.\n\nWhen only single character classes are presented,\nor several classes which are `unions`, `:not_in`\ncan be preserved.\n\nWhen intersections and differences are required,\nthe rnages must be both reduced and expanded in\norder for this set operations to complete.","ref":"Unicode.Set.Operation.html#reduce/1"},{"type":"function","title":"Unicode.Set.Operation.symmetric_difference/2","doc":"Returns the difference of two lists of\n2-tuples representing codepoint ranges.\n\nThe result is a single list of codepoint\nranges that represents the codepoints\nthat are in either of the two lists but\nnot both.","ref":"Unicode.Set.Operation.html#symmetric_difference/2"},{"type":"function","title":"Unicode.Set.Operation.traverse/2","doc":"Prewalks the expanded AST from a parsed\nUnicode Set invoking a function on each\ncodepoint range in the set.","ref":"Unicode.Set.Operation.html#traverse/2"},{"type":"function","title":"Unicode.Set.Operation.traverse/3","doc":"","ref":"Unicode.Set.Operation.html#traverse/3"},{"type":"function","title":"Unicode.Set.Operation.union/2","doc":"Merges two lists of 2-tuples representing\nranges of codepoints.  The result is a\nsingle list of 2-tuple codepoint ranges\nthat includes all codepoint from the\ntwo lists.","ref":"Unicode.Set.Operation.html#union/2"},{"type":"module","title":"Unicode.Set.Search","doc":"","ref":"Unicode.Set.Search.html"},{"type":"function","title":"Unicode.Set.Search.build_search_tree/1","doc":"","ref":"Unicode.Set.Search.html#build_search_tree/1"},{"type":"function","title":"Unicode.Set.Search.extract_and_expand_string_ranges/1","doc":"","ref":"Unicode.Set.Search.html#extract_and_expand_string_ranges/1"},{"type":"function","title":"Unicode.Set.Search.member?/2","doc":"","ref":"Unicode.Set.Search.html#member?/2"},{"type":"function","title":"Unicode.Set.Search.string_member?/2","doc":"","ref":"Unicode.Set.Search.html#string_member?/2"},{"type":"exception","title":"Unicode.Set.ParseError","doc":"Exception raised when an a Unicode Set cannot\nbe parsed","ref":"Unicode.Set.ParseError.html"},{"type":"extras","title":"Unicode Set","doc":"# Unicode Set\n\n![Build Status](https://api.cirrus-ci.com/github/elixir-unicode/unicode_set.svg)\n[![Hex.pm](https://img.shields.io/hexpm/v/unicode_set.svg)](https://hex.pm/packages/unicode_set)\n[![Hex.pm](https://img.shields.io/hexpm/dw/unicode_set.svg?)](https://hex.pm/packages/unicode_set)\n[![Hex.pm](https://img.shields.io/hexpm/l/unicode_set.svg)](https://hex.pm/packages/unicode_set)\n\nA [Unicode Set](https://unicode-org.github.io/icu/userguide/strings/unicodeset.html) is a representation of a set of Unicode characters or character strings. The contents of that set are specified by patterns or by building them programmatically. This library implements parsing of unicode sets, resolving them to a list of codepoints and matching a given codepoint to that list.  This expansion supports the following public API:\n\n* `Unicode.Set.match?/2` which is a macro that matches a codepoint to a unicode set.\n* `Unicode.Regex.compile/2` which pre-processes a regex string expanding unicode sets into a regex executable by the `Regex` module.\n* `Unicode.Set.to_utf8_char/1` that converts a unicode set into a form usable with [nimble_parsec](https://hex.pm/packages/nimble_parsec)\n* `Unicode.Set.compile_pattern/1` which converts a unicode set into a string that is then compiled with `:binary.compile_pattern/1`.\n\nThe implementation conforms closely to the [Unicode Set specification](https://unicode.org/reports/tr35/#Unicode_Sets) but currently omits support for the `\\N{codepoint_name}` syntax.\n\n<!-- MDOC -->","ref":"readme.html"},{"type":"extras","title":"Usage - Unicode Set","doc":"","ref":"readme.html#usage"},{"type":"extras","title":"Function guards - Unicode Set","doc":"This is helpful in defining [function guards](https://hexdocs.pm/elixir/patterns-and-guards.html#guards). For example:\n```elixir\ndefmodule Guards do\n  require Unicode.Set\n\n  # Define a guard that checks if a codepoint is a unicode digit\n  defguard digit?(x) when Unicode.Set.match?(x, \"[[:Nd:]]\")\nend\n\ndefmodule MyModule do\n  require Guards\n\n  # Define a function using the previously defined guard\n  def my_function(< >) when Guards.digit?(x) do\n    IO.puts \"Its a digit!\"\n  end\n\n  # Define a guard directly on the function\n  def my_other_function_(< >) when Unicode.Set.match?(x, \"[[:Nd:]]\") do\n    IO.puts \"Its also a digit!\"\n  end\nend\n```","ref":"readme.html#function-guards"},{"type":"extras","title":"Generating compiled patterns for String matching - Unicode Set","doc":"`String.split/3` and `String.replace/3` allow for patterns and [compiled patterns](http://erlang.org/doc/man/binary.html#compile_pattern-1) to be used with compiled patterns being the more performant approach.  Unicode Set supports the generation of patterns and compiled patterns:\n```elixir\niex> pattern = Unicode.Set.compile_pattern!(\"[[:digit:]]\")\niex> list = String.split(\"abc1def2ghi3jkl\", pattern)\n[\"abc\", \"def\", \"ghi\", \"jkl\"]\n```","ref":"readme.html#generating-compiled-patterns-for-string-matching"},{"type":"extras","title":"Generating NimbleParsec ranges - Unicode Set","doc":"The parser generator [nimble_parsec](https://hex.pm/packages/nimble_parsec) allows a list of codepoint ranges as parameters to several combinators. Unicode Set can generate such ranges:\n```\niex> Unicode.Set.to_utf8_char!(\"[[^abcd][mnb]]\")\n[98, 109..110, {:not, 97..100}]\n```\nThis can be used as shown in the following example:\n```elixir\ndefmodule MyCombinators do\n  import NimbleParsec\n\n  @digit_list = Unicode.Set.to_utf8_char!(\"[[:digit:]]\")\n  def unicode_digit do\n    utf8_char(@digit_list)\n    |> label(\"a digit in any Unicode script\")\n  end\nend\n```","ref":"readme.html#generating-nimbleparsec-ranges"},{"type":"extras","title":"Compiling extended regular expressions - Unicode Set","doc":"The `Regex` module supports a limited set of Unicode Sets. The `Unicode.Regex` module provides `compile/2` and `compile!/2` functions that have the same arguments and compatible functionality with `Regexp.compile/2` other that they pre-process the regular expression, expanding any Unicode Sets. This makes it simple to incorporate Unicode Sets in regular expressions.\n\nAll Unicode Sets are expanded, even those that are known to `Regex.compile/2` since the erlang `:re` module upon `Regex` is based does not always keep pace with Unicode releases.\n\nFor example:\n\n```elixir\niex> Unicode.Regex.compile(\"\\\\p{Zs}\")\n{:ok, ~r/[\\x{20}\\x{A0}\\x{1680}\\x{2000}-\\x{200A}\\x{202F}\\x{205F}\\x{3000}]/u}\n\niex> Unicode.Regex.compile(\"[:graphic:]\")\n{:ok,\n ~r/[\\x{20}-\\x{7E}\\x{A0}-\\x{AC}\\x{AE}-\\x{377}\\x{37A}-\\x{37F}...]/u}\n```","ref":"readme.html#compiling-extended-regular-expressions"},{"type":"extras","title":"Other Examples - Unicode Set","doc":"These examples show how to combine sets (union, difference and intersection) to deliver a flexible targeting of the required match.\n\n```elixir\n# The character \"๓\" is the thai digit `1`\niex> Unicode.Set.match? ?๓, \"[[:digit:]]\"\ntrue\n\n# Set operations allow union, insersection and difference\n# This example matches on digits, but not the Thai script\niex> Unicode.Set.match? ?๓, \"[[:digit:]-[:thai:]]\"\nfalse\n```","ref":"readme.html#other-examples"},{"type":"extras","title":"Compile time parsing - Unicode Set","doc":"As much work as possible is done at compile time in order to deliver good performance. The macro `Unicode.Set.match?/2` parses the unicode set, expands the require codepoints and generates guard clauses at compile time. The resulting code is a simple set of boolean operators that executes quickly at runtime.","ref":"readme.html#compile-time-parsing"},{"type":"extras","title":"Supported Unicode properties - Unicode Set","doc":"This version of `Unicode Set` supports the following enumerable unicode properties in unicode sets:\n\n* `script` such as `[:script=arabic:]`, `\\p{script=arabic}` or `[:arabic:]`\n* `block` such as `[:block=sudanese:]`, `\\p{block=sudanese}`, `\\p{IsSudanese}` or `[:IsSudanese:]`\n* `general category` such as `[:Lu:]`, `\\p{Lu}`, `[:gc=Lu:]` or `[:general category=Lu:]`\n* `combining class` such as `[:ccc=230:]`\n\nIn addition, the following boolean properties are supported. These are expressed as `[:white space:]` or `\\p{White Space}`.\n\n\nProperty   | Property | Property | Property\n---------- | -------- | -------- | ----------\nalphabetic | ascii_hex_digit | bidi_control | cased\n changes_when_casemapped | changes_when_lowercased |  changes_when_titlecased | changes_when_uppercased\ndash | default_ignorable_code_point |  deprecated |  diacritic\nextender | grapheme_base |  grapheme_extend |  grapheme_link\nhex_digit | hyphen |  id_continue |  id_start\nideographic | ids_binary_operator |  ids_trinary_operator |  join_control\nlogical_order_exception | lowercase |  math |  noncharacter_code_point\nother_alphabetic | other_default_ignorable_code_point |  other_grapheme_extend |  other_id_continue\nother_id_start |  other_lowercase | other_math |  other_uppercase\npattern_syntax |  pattern_white_space | prepended_concatenation_mark |  quotation_mark\nradical |  regional_indicator | sentence_terminal |  soft_dotted\nterminal_punctuation |  unified_ideograph | uppercase |  variation_selector\nwhite_space |  xid_continue | xid_start | changes_when_casefolded\n\nIn all cases, property names and property values may include whitespace and mixed case notation.","ref":"readme.html#supported-unicode-properties"},{"type":"extras","title":"General Categories - Unicode Set","doc":"Abbreviation\t      | Long Form\n------------------- | --------------------------------------\nL  |\tLetter\nLu |\tUppercase Letter\nLl |\tLowercase Letter\nLt |\tTitlecase Letter\nLm |\tModifier Letter\nLo |\tOther Letter\nM  |\tMark\nMn |\tNon-Spacing Mark\nMc |\tSpacing Combining Mark\nMe |\tEnclosing Mark\nN  |\tNumber\nNd |\tDecimal Digit Number\nNl |\tLetter Number\nNo |\tOther Number\nS  |\tSymbol\nSm |\tMath Symbol\nSc |\tCurrency Symbol\nSk |\tModifier Symbol\nSo |\tOther Symbol\nP  |\tPunctuation\nPc |\tConnector Punctuation\nPd |\tDash Punctuation\nPs |\tOpen Punctuation\nPe |\tClose Punctuation\nPi |\tInitial Punctuation\nPf |\tFinal Punctuation\nPo |\tOther Punctuation\nZ  |\tSeparator\nZs |\tSpace Separator\nZl |\tLine Separator\nZp |\tParagraph Separator\nC  |\tOther\nCc |\tControl\nCf |\tFormat\nCs |\tSurrogate\nCo |\tPrivate Use\nCn |\tUnassigned\n\nDerived Categories\t| Long Form\n------------------- | --------------------------------------\nAny                 | Any\tall code points\t[\\u{0}-\\u{10FFFF}]\nAssigned            | Assigned\tall assigned characters meaning\t`\\P{Cn}`\nASCII               | ASCII\tall ASCII characters\t[\\u{0}-\\u{7F}]","ref":"readme.html#general-categories"},{"type":"extras","title":"Compatibility Property Names - Unicode Set","doc":"Property  | Unicode Category           | Comments\n--------- | -------------------------- | -----------\nalpha\t    | `\\p{Alphabetic}`           | Alphabetic includes more than gc = Letter. Note that combining marks (Me, Mn, Mc) are required for words of many languages. While they could be applied to non-alphabetics, their principal use is on alphabetics. Alphabetic should not be used as an approximation for word boundaries: see `word` below.\nlower\t    | `\\p{Lowercase}`\t             | Lowercase includes more than gc = Lowercase_Letter (Ll).\nupper\t    | `\\p{Uppercase}`\t             | Uppercase includes more than gc = Uppercase_Letter (Lu).\npunct\t    | `\\p{gc=Punctuation} \\p{gc=Symbol} - \\p{alpha}` | Punctuation and symbols.\ndigit     |\t`\\p{gc=Decimal_Number}`\t     | [0..9]\tNon-decimal numbers (like Roman numerals) are normally excluded.\nxdigit    | `\\p{gc=Decimal_Number} \\p{Hex_Digit}`\t| [0-9 A-F a-f]\tHex_Digit contains 0-9 A-F, fullwidth and halfwidth, upper and lowercase.\nalnum     |\t`\\p{alpha} \\p{digit}`\t       | Simple combination of other properties\nspace     |\t`\\p{Whitespace}`\t|\nblank\t    | `\\p{gc=Space_Separator} \\N{CHARACTER TABULATION}`\t| \"horizontal\" whitespace: space separators plus U+0009 tab.\ncntrl\t    | `\\p{gc=Control} `            | The characters in \\p{gc=Format} share some, but not all aspects of control characters. Many format characters are required in the representation of plain text.\ngraph\t    | `[^\\p{space} \\p{gc=Control} \\p{gc=Surrogate} \\p{gc=Unassigned}]`\t| Warning: the set shown here is defined by excluding space, controls, and so on with ^.\nprint\t    | `\\p{graph} \\p{blank} -- \\p{cntrl}`\t| Includes graph and space-like characters.\nword      | `\\p{alpha} \\p{gc=Mark} \\p{digit} \\p{gc=Connector_Punctuation} \\p{Join_Control}`\t|\tThis is only an approximation to Word Boundaries. The Connector Punctuation is added in for programming language identifiers, thus adding `_` and similar characters.","ref":"readme.html#compatibility-property-names"},{"type":"extras","title":"Additional Derived properties - Unicode Set","doc":"In addition to the Unicode properties, some additional properties are also defined for convenience. These properties related to quote marks and are:\n\n* `quote_mark`\n* `quote_mark_left`\n* `quote_mark_right`\n* `quote_mark_ambidextrous`\n* `quote_mark_single`\n* `quote_mark_double`\n\nAs above these properties can be expressed in mixed case with spaces and underscores inserted for readability.  They can be used in the same way as any Unicode property name.","ref":"readme.html#additional-derived-properties"},{"type":"extras","title":"Example Unicode Sets - Unicode Set","doc":"Here are a few examples of sets. Although elements of the syntax appear similar to regular expressions, unicode sets only expresses one or more ranges of unicode codepoints.\n\nPattern\t              | Description\n--------------------- | -----------------------------------------------------------\n`[a-z]`               | The lower case letters `a` through `z`\n`[abc123]`            | The six characters `a,b,c,1,2` and `3`\n`[\\p{Letter}]`        | All characters with the Unicode General Category of Letter","ref":"readme.html#example-unicode-sets"},{"type":"extras","title":"String Values - Unicode Set","doc":"In addition to being a set of characters (of Unicode code points), a UnicodeSet may also contain string values. Conceptually, the UnicodeSet is always a set of strings, not a set of characters, although in many common use cases the strings are all of length one, which reduces to being a set of characters.\n\nThis concept can be confusing when first encountered, probably because similar set constructs from other environments (regular expressions) can only contain characters.","ref":"readme.html#string-values"},{"type":"extras","title":"Unicode Set Patterns - Unicode Set","doc":"Patterns are a series of characters bounded by square brackets that contain lists of characters and Unicode property sets. Lists are a sequence of characters that may have ranges indicated by a '-' between two characters, as in \"a-z\". The sequence specifies the range of all characters from the left to the right, in Unicode order. For example, `[a c d-f m]` is equivalent to `[a c d e f m]`. Whitespace can be freely used for clarity as `[a c d-f m]` means the same as `[acd-fm]`.\n\nUnicode property sets are specified by a Unicode property, such as [:Letter:]. For a list of supported properties, see the [Properties](#supported-unicode-properties) section. For details on the use of short vs. long property and property value names, see the end of this section. The syntax for specifying the property names is an extension of either POSIX or Perl syntax with the addition of `=value`. For example, you can match letters by using the POSIX syntax `[:Letter:]`, or by using the Perl-style syntax `\\p{Letter}`. The type can be omitted for the `Category` and `Script` properties, but is required for other properties.\n\nThe table below shows the two kinds of syntax: POSIX and Perl style. Also, the table shows the \"Negative\", which is a property that excludes all characters of a given kind. For example, `[:^Letter:]` matches all characters that are not `[:Letter:]`.\n\nStyle              | Positive\t        | Negative\n------------------ | ---------------- | ----------------\nPOSIX-style Syntax | [:type=value:]   |\t[:^type=value:]\nPerl-style Syntax  |\t\\p{type=value}\t| \\P{type=value}\n\nThese following low-level lists or properties then can be freely combined with the normal set operations (union, inverse, difference, and intersection):\n\nExample\t                       | Meaning\n------------------------------ | -----------------------------------------------------------\n`A B\t[[:letter:] [:number:]]` | To union two sets A and B, simply concatenate them\n`A & B\t[[:letter:] & [a-z]]`  | To intersect two sets A and B, use the '&' operator.\n`A - B\t[[:letter:] - [a-z]]`\t | To take the set-difference of two sets A and B, use the '-' operator.\n`[^A]\t[^a-z]`\t                 | To invert a set A, place a `^` immediately after the opening `[`. Note that the complement only affects code points, not string values. In any other location, the `^` does not have a special meaning.","ref":"readme.html#unicode-set-patterns"},{"type":"extras","title":"Precedence - Unicode Set","doc":"The binary operators of union, intersection, and set-difference have equal precedence and bind left-to-right. Thus the following are equivalent:\n\n* `[[:letter:] - [a-z] [:number:] & [\\u0100-\\u01FF]]`\n* `[[[[[:letter:] - [a-z]] [:number:]] & [\\u0100-\\u01FF]]`\n\nAnother example is that the set `[[ace][bdf] - [abc][def]]` is not the empty set, but instead the set `[def]`. That is because the syntax corresponds to the following UnicodeSet operations:\n\n1. start with `[ace]`\n2. union `[bdf]`  -- we now have `[abcdef]`\n3. subtract `[abc]` -- we now have `[def]`\n4. union `[def]` -- no effect, we still have `[def]`\n\nThis only really matters where there are the difference and intersection operations, as the union operation is commutative. To make sure that the - is the main operator, add brackets to group the operations as desired, such as `[[ace][bdf] - [[abc][def]]]`.\n\nAnother caveat with the `&` and `-` operators is that they operate between sets. That is, they must be immediately preceded and immediately followed by a set. For example, the pattern `[[:Lu:]-A]` is illegal, since it is interpreted as the set [:Lu:] followed by the incomplete range -A. To specify the set of uppercase letters except for `A`, enclose the `A` in a set: `[[:Lu:]-[A]]`.","ref":"readme.html#precedence"},{"type":"extras","title":"Examples - Unicode Set","doc":"* `[a]`\tThe set containing 'a'\n* `[a-z]`\tThe set containing 'a' through 'z' and all letters in between, in Unicode order\n* `[^a-z]`\tThe set containing all characters but 'a' through 'z', that is, U+0000 through 'a'-1 and 'z'+1 through U+FFFF\n* `[[pat1][pat2]]`\tThe union of sets specified by pat1 and pat2\n* `[[pat1]&[pat2]]`\tThe intersection of sets specified by pat1 and pat2\n* `[[pat1]-[pat2]]`\tThe asymmetric difference of sets specified by pat1 and pat2\n* `[:Lu:]`\tThe set of characters belonging to the given Unicode category; in this case, Unicode uppercase letters. The long form for this is `[:UppercaseLetter:]`.\n* `[:L:]`\tThe set of characters belonging to all Unicode categories starting with 'L', that is, `[[:Lu:][:Ll:][:Lt:][:Lm:][:Lo:]]`. The long form for this is `[:Letter:]`.","ref":"readme.html#examples"},{"type":"extras","title":"String Values in Sets - Unicode Set","doc":"String values are enclosed in `{`curly brackets`}`.\n\nSet expression\t    | Description\n------------------- | --------------------------------------\n`[abc{def}]`\t      | A set containing four members, the single characters a, b and c, and the string “def”\n`[{abc}{def}]`      |\tA set containing two members, the string “abc” and the string “def”.\n`[{a}{b}{c}][abc]`\t| These two sets are equivalent. Each contains three items, the three individual characters `a`, `b` and `c`. A `{string}` containing a single character is equivalent to that same character specified in any other way.","ref":"readme.html#string-values-in-sets"},{"type":"extras","title":"Character Quoting and Escaping in Unicode Set Patterns - Unicode Set","doc":"","ref":"readme.html#character-quoting-and-escaping-in-unicode-set-patterns"},{"type":"extras","title":"Single Quote - Unicode Set","doc":"Two single quotes represents a single quote, either inside or outside single quotes.\n\nText within single quotes is not interpreted in any way (except for two adjacent single quotes). It is taken as literal text (special characters become non-special).\n\nThese quoting conventions for ICU UnicodeSets differ from those of regular expression character set expressions. In regular expressions, single quotes have no special meaning and are treated like any other literal character.","ref":"readme.html#single-quote"},{"type":"extras","title":"Backslash Escapes - Unicode Set","doc":"Outside of single quotes, certain backslashed characters have special meaning. Note that these are escapes processed by Unicode Set (this library) and therefore require `\\\\\\\\` to be entered as a prefix. [Elixir also provides similar escapes](https://elixir-lang.org/getting-started/sigils.html#interpolation-and-escaping-in-sigils) as native part of its string processing and Elixir's escapes are to be preferred where possible.\n\nEscape         | Description\n-------------- | -------------------------------------------------\n\\uhhhh\t       | Exactly 4 hex digits; h in [0-9A-Fa-f]\n\\Uhhhhhhhh\t   | Exactly 8 hex digits\n\\xhh\t         | 1-2 hex digits\n\nCertain other escapes are native to Elixir and are applicable in Unicode Sets they are in any Elixir string:\n\nEscape         | Description\n-------------- | -------------------------------------------------\n\\a\t           | U+0007 (BELL)\n\\b\t           | U+0008 (BACKSPACE)\n\\t\t           | U+0009 (HORIZONTAL TAB)\n\\n\t           | U+000A (LINE FEED)\n\\v\t           | U+000B (VERTICAL TAB)\n\\f\t           | U+000C (FORM FEED)\n\\r\t           | U+000D (CARRIAGE RETURN)\n\\\\\t           | U+005C (BACKSLASH)\n\\xDD           | represents a single byte in hexadecimal (such as `\\x13`)\n\\uDDDD and \\u{D...} | represents a Unicode codepoint in hexadecimal (such as `\\u{1F600}`)\n\nAnything else following a backslash is mapped to itself, except in an environment where it is defined to have some special meaning. For example, `\\p{Lu}` is the set of uppercase letters in a Unicode Set.\n\nAny character formed as the result of a backslash escape loses any special meaning and is treated as a literal. In particular, note that `\\u` and `\\U` escapes create literal characters.","ref":"readme.html#backslash-escapes"},{"type":"extras","title":"Whitespace - Unicode Set","doc":"Whitespace (as defined by the specification) is ignored unless it is quoted or backslashed.","ref":"readme.html#whitespace"},{"type":"extras","title":"Property Values - Unicode Set","doc":"The following property value variants are recognized:\n\nFormat\t  | Example                           | Description\n--------- | --------------------------------- | ----------------------------------------------\nshort\t    | Lu                                | omits the type (used to prevent ambiguity and only allowed with the Category and Script properties)\nmedium\t  | gc=Lu                             | uses an abbreviated type and value\nlong\t    | General_Category=Uppercase_Letter | uses a full type and value\n\nIf the type or value is omitted, then the equals sign is also omitted. The short style is only\nused for Category and Script properties because these properties are very common and their omission is unambiguous.\n\nIn actual practice, you can mix type names and values that are omitted, abbreviated, or full. For example, if Category=Unassigned you could use what is in the table explicitly, `\\p{gc=Unassigned}`, `\\p{Category=Cn}`, or `\\p{Unassigned}`.\n\nWhen these are processed, case and whitespace are ignored so you may use them for clarity, if desired. For example, `\\p{Category = Uppercase Letter}` or `\\p{Category = uppercase letter}`.\n\n<!-- MDOC -->","ref":"readme.html#property-values"},{"type":"extras","title":"Installation - Unicode Set","doc":"To install, add the package `unicode_set` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:unicode_set, \"~> 1.0\"}\n  ]\nend\n```\n\nDocumentation can be found at [https://hexdocs.pm/unicode_set](https://hexdocs.pm/unicode_set).","ref":"readme.html#installation"},{"type":"extras","title":"License","doc":"# License\n\nCopyright 2019 Kip Cole\n\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use this file except in\ncompliance with the License. You may obtain a copy of the License at\n\n    http://www.apache.org/licenses/LICENSE-2.0\n\nUnless required by applicable law or agreed to in writing, software distributed under the License\nis distributed on an \"AS IS\" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\nimplied. See the License for the specific language governing permissions and limitations under the\nLicense.","ref":"license.html"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAs of `unicode_set` version 1.4.0, Elixir 1.12 or later is required.","ref":"changelog.html"},{"type":"extras","title":"Unicode Set 1.5.0 - Changelog","doc":"This is the changelog for Unicode Set 1.5.0 released on March 29th, 2025. For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-unicode/unicode_set/tags)","ref":"changelog.html#unicode-set-1-5-0"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Converts all compile-time regex compilation to runtime to be compatible with OTP 28. Performance implications are not yet known.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Unicode Set 1.4.1 - Changelog","doc":"This is the changelog for Unicode Set 1.4.1 released on January 1st, 2025. For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-unicode/unicode_set/tags)","ref":"changelog.html#unicode-set-1-4-1"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Work around the Elixir type checker for now.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Unicode Set 1.4.0 - Changelog","doc":"This is the changelog for Unicode Set 1.4.0 released on May 26th, 2024. For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-unicode/unicode_set/tags)","ref":"changelog.html#unicode-set-1-4-0"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix warnings for Elixir 1.17. Thanks to @alco for the PR.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Unicode Set 1.3.0 - Changelog","doc":"This is the changelog for Unicode Set 1.3.0 released on February 18th, 2023. For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-unicode/unicode_set/tags)","ref":"changelog.html#unicode-set-1-3-0"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Correct the code examples in README.md.  Thanks to @DianaOlympos for the PR. Closes #9.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add `Unicode.Set.compile_pattern!/1` to accompany `Unicode.Set.compile_pattern/1`.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Unicode Set 1.2.0 - Changelog","doc":"This is the changelog for Unicode Set 1.2.0 released on September 15th, 2022. For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-unicode/unicode_set/tags)","ref":"changelog.html#unicode-set-1-2-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Update parsing code to ensure compatibility against future deprecations. Thanks to @josevalim.\n\n* Fix library name in doc links. Thanks to @zmaril for the PR.\n\n* Update dependencies. Thanks to @kianmeng.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Unicode Set 1.1.0 - Changelog","doc":"This is the changelog for Unicode Set 1.1.0 released on September 15th, 2021.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-unicode/unicode_set/tags)","ref":"changelog.html#unicode-set-1-1-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* `ex_unicode` is renamed to `unicode` in collaboration with @Qqwy and therefore this release updates the dependency name.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Unicode Set 1.0.0 - Changelog","doc":"This is the changelog for Unicode Set 1.0.0 released on September 14th, 2021.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-unicode/unicode_set/tags)","ref":"changelog.html#unicode-set-1-0-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Update to use [Unicode 14](https://unicode.org/versions/Unicode14.0.0)","ref":"changelog.html#enhancements"},{"type":"extras","title":"Unicode Set 0.13.1 - Changelog","doc":"This is the changelog for Unicode Set 0.13.1 released on May 25th, 2021.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-unicode/unicode_set/tags)","ref":"changelog.html#unicode-set-0-13-1"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Update dependency configuration to mark `ex_doc` and `benchee` as optional.  Thanks to @fireproofsocks.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Unicode Set 0.13.0 - Changelog","doc":"This is the changelog for Unicode Set 0.13.0 released on April 4th, 2021.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-unicode/unicode_set/tags)","ref":"changelog.html#unicode-set-0-13-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Adds `Unicode.Set.to_generate_matches/1` that returns a tuple whose first element is the AST of a guard clause and the second element is a list of strings. This function is marked private and is implemented to support [unicode_transform](https://hex.pm/unicode_transform) which uses this information to generate optimised code for matching unicode sets in a `case` expression.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Unicode Set 0.12.0 - Changelog","doc":"This is the changelog for Unicode Set 0.12.0 released on February 23rd, 2021.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-unicode/unicode_set/tags)","ref":"changelog.html#unicode-set-0-12-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Adds support for \"isBlockName\" Perl and POSIX regex syntax. Used in a regex as `[[:isLatin1]]` or `\\p{isLatin1}` or their inverse forms `[[:^isLatin1]]` and `\\P{isLatin1}`.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Unicode Set 0.11.0 - Changelog","doc":"This is the changelog for Unicode Set 0.11.0 released on October 5th, 2020.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-unicode/unicode_set/tags)","ref":"changelog.html#unicode-set-0-11-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add recursively defined sets to support compatibility with Posix classes. See `Unicode.Set.Property`.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix various bugs in set operations for `Union`, `Difference`, `Intersection` abd `Complement`\n\n* Correctly parse and interpret set complements such as `[^[:^Sc:]]` and more complex sets such as `[^[[:Sc:]-[:^Lu:]]]`","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Unicode Set 0.10.0 - Changelog","doc":"This is the changelog for Unicode Set 0.10.0 released on October 2nd, 2020.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-unicode/unicode_set/tags)","ref":"changelog.html#unicode-set-0-10-0"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix list composition in `Unicode.Set.to_uft8_char/1`","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Unicode Set 0.9.0 - Changelog","doc":"This is the changelog for Unicode Set 0.9.0 released on October 2nd, 2020.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-unicode/unicode_set/tags)","ref":"changelog.html#unicode-set-0-9-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Support `nimble_parsec` version 1.x. Thanks to @josevalim for the PR.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Unicode Set 0.8.0 - Changelog","doc":"This is the changelog for Unicode Set 0.8.0 released on July 12th, 2020.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-unicode/unicode_set/tags)","ref":"changelog.html#unicode-set-0-8-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Rewrite `Unicode.Regex` module to better extract character classes, process unicode sets and build Elixir regexs. Now also supports string ranges.\n\n* Supports the property `East Asian Width` (short name `ea`) which is required for implementing the Unicode segmentation algorithms.  Also bumps the minimum requirement for [ex_unicode version 1.8](https://hex.pm/packages/ex_unicode/1.8.0).","ref":"changelog.html#enhancements"},{"type":"extras","title":"Unicode Set 0.7.0 - Changelog","doc":"This is the changelog for Unicode Set v.07.0 released on May 18th, 2020.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-unicode/unicode_set/tags)","ref":"changelog.html#unicode-set-0-7-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add `Unicode.Set.character_class/1` which returns a string compatible with `Regex.compile/2`. This supports the idea of expanded Unicode Sets being used in standard Elixir/erlang regular expressions and will underpin implementation of Unicode Transforms in the package `unicode_transform`\n\n* Add `Unicode.Regex.compile/2` to pre-process a regex to expand Unicode Sets and the compile it with `Regex.compile/2`.  `Unicode.Regex.compile!/2` is also added.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fixes a bug whereby a Unicode Set intersection would fail with a character class that starts at the same codepoint as the Unicode set.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Unicode Set 0.6.0 - Changelog","doc":"This is the changelog for Unicode Set v.06.0 released on May 13th, 2020.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-unicode/unicode_set/tags)","ref":"changelog.html#unicode-set-0-6-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Unicode sets are now a `%Unicode.Set{}` struct\n\n* Add `Unicode.Set.Sigil` implementing `sigil_u`\n\n* Add support for `String.Chars` and `Inspect` protocols","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fixes parsing sets to ignore non-encoded whitespace\n\n* Fixes intersection and difference set operations for sets that include string ranges like `{abc}`","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Unicode Set 0.5.1 - Changelog","doc":"This is the changelog for Unicode Set v.05.1 released on March 14th, 2020.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-unicode/unicode_set/tags)","ref":"changelog.html#unicode-set-0-5-1"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Compacts tuple-ranges in order to minimize the number of generated clauses in guards. Requires at least `ex_unicode` version 1.5.0.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Unicode Set 0.5.0 - Changelog","doc":"This is the changelog for Unicode Set v.05.0 released on March 11th, 2020.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-unicode/unicode_set/tags)","ref":"changelog.html#unicode-set-0-5-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Updates `ex_unicode` to version `1.4.0` which includes support for [Unicode version 13.0](http://blog.unicode.org/2020/03/announcing-unicode-standard-version-130.html) as well as support for several derived categories related to quote marks.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Unicode Set 0.4.2 - Changelog","doc":"This is the changelog for Unicode Set v.04.2 released on February 25th, 2020.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-unicode/unicode_set/tags)","ref":"changelog.html#unicode-set-0-4-2"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Allow `\\n`, `\\t` and `\\r`, `\\s` as part of character classes","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Unicode Set 0.4.1 - Changelog","doc":"This is the changelog for Unicode Set v.04.1 released on January 8th, 2020.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-unicode/unicode_set/tags)","ref":"changelog.html#unicode-set-0-4-1"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix `Unicode.Set.Operation.difference/2` when one list is wholly contained within another","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Unicode Set 0.4.0 - Changelog","doc":"This is the changelog for Unicode Set v.04.0 released on November 27th, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-unicode/unicode_set/tags)","ref":"changelog.html#unicode-set-0-4-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Bump to [ex_unicode](https://hex.pm/packages/ex_unicode) to version 1.3.0 to support an expanded set of properties resolved by `unicode_set`.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Unicode Set 0.3.0 - Changelog","doc":"This is the changelog for Unicode Set v.03.0 released on November 26th, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-unicode/unicode_set/tags)","ref":"changelog.html#unicode-set-0-3-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Support string ranges expressed as `{abc}` or `{abc}-{def}`\n\n* Note that the supported proporties in this release are `script`, `block`, `category` and `combining class`.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Unicode Set 0.2.0 - Changelog","doc":"This is the changelog for Unicode Set v.02.0 released on November 24th, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-unicode/unicode_set/tags)","ref":"changelog.html#unicode-set-0-2-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add `Unicode.Set.compile_pattern/1` and `Unicode.Set.pattern/1` to generate patterns and compiled patterns compatible with `String.split/3` and `String.replace/3`.\n\n* Add `Unicode.Set.utf8_char/1` that generates a list of codepoint ranges compatible with [nimble_parsec](https://hex.pm/packages/nimble_parsec) combinators.\n\nSet the README for example usage.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Unicode Set 0.1.0 - Changelog","doc":"This is the changelog for Unicode Set v.01.0 released on November 23rd, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-unicode/unicode_set/tags)\n\nInitial release.","ref":"changelog.html#unicode-set-0-1-0"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}